@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2014 Codership Oy <info@codership.com>
+ * Copyright (C) 2010-2017 Codership Oy <info@codership.com>
  */
 
 #include "write_set.hpp"
@@ -18,11 +18,30 @@ namespace
 {
     class TestEnv
     {
+        class GCache_setup
+        {
+        public:
+            GCache_setup(gu::Config& conf) : name_("write_set_test.gcache")
+            {
+                conf.set("gcache.name", name_);
+                conf.set("gcache.size", "1M");
+                log_info << "conf for gcache: " << conf;
+            }
+
+            ~GCache_setup()
+            {
+                unlink(name_.c_str());
+            }
+        private:
+            std::string const name_;
+        };
+
     public:
 
         TestEnv() :
             conf_   (),
             init_   (conf_, NULL, NULL),
+            gcache_setup_(conf_),
             gcache_ (conf_, "."),
             gcs_    (conf_, gcache_),
             thd_    (gcs_,  gcache_)
@@ -35,9 +54,9 @@ namespace
 
     private:
 
-// REMOVE    galera::Certification::register_params(conf);
         gu::Config         conf_;
         galera::ReplicatorSMM::InitConfig init_;
+        GCache_setup       gcache_setup_;
         gcache::GCache     gcache_;
         galera::DummyGcs   gcs_;
         galera::ServiceThd thd_;
@@ -322,7 +341,7 @@ START_TEST(test_mapped_buffer)
 END_TEST
 
 static TrxHandle::LocalPool
-lp(TrxHandle::LOCAL_STORAGE_SIZE, 4, "ws_local_pool");
+lp(TrxHandle::LOCAL_STORAGE_SIZE(), 4, "ws_local_pool");
 
 static TrxHandle::SlavePool
 sp(sizeof(TrxHandle), 4, "ws_slave_pool");
